// Copyright 2014-2015, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved.

package com.sas.deployment
import groovy.transform.InheritConstructors

import com.sas.gate.CodeAccessor
import com.sas.gate.api.PipeCode
import com.sas.gate.api.template.*

/**
   <overview>Deployment code for SAS Visual Scenario Designer Mid-tier (scdmid)</overview>
   <p>
      This code configures the deployment of scdmid using what corresponds to
      the following Ant targets:
      <ul class='nodot'>
         <li>configure</li>
         <li>buildWebapps</li>
         <li>deployWebapps</li>
         <li>updateconfigure</li>
         <li>changehost</li>
         <li>updatepassword</li>
         <li>unconfigure</li>
      </ul>
   </p>

   @since      14w19
   @version    15w04
   @author     Prashant Ghorpade
   @author     Steve Amerige
   @see        #SOURCE#
*/

@InheritConstructors
class SCDMID extends CodeAccessor
{
   // ===================================================== GLOBAL

   /**
      Identifier

      @param id <type>String</type>: The scdmid name
   */
   String id = 'SCDMID'

   /**
      Callback for hook: <hook>properties &gt; init</hook>

      @param t__properties_get   <type>ArrayList&lt;String&gt;</type>: get object properties on each dependson property

   <ul class='nodot'>
      <li>levconfig</li>
      <li>workspacesrv</li>
   </ul>
   */
   def properties_init =
   {

      pass.with {
         debug = ['debug']

         _scdmidinstalldir                      = api.'scdmid.install.dir'
         _rootAppName                           = api.'scdmid.archive.name'
         _releaseNumber                         = api.'scdmid.release.number'
         _shortNormalizedLegalName              = api.'scdmid.normalized.short.name'

         t__properties_get =
         [
            'scdmid.dependson.webappsrv.fqid',
            'scdmid.dependson.webinfpltfm.fqid',
            'scdmid.dependson.webinfdsvrc.fqid',
            'scdmid.dependson.bisrvmid.fqid',
            'scdmid.dependson.scdsvrc.fqid',
            'scdmid.dependson.bihpgrdc.fqid',
          ]
      }
   }

   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param filters <type>Map&lt;String&gt;</type>: tokens for string substitution
      @param t__setobjectproperties    <type>ArrayList&lt;String&gt;</type>: Properties that will be stored in the configured objects text store
      @param t__createdirectories <type>Map&lt;String&gt;</type>: directories to be created in configure target<br>
            <indent><key>prefix</key>- <type>String</type>: base directory to create<br>
            <key>entries</key>- <type>ArrayList&lt;String&gt;</type>: directories and subdirectories to be created<br></indent>
      <pre>
   /install/config/Lev1/Applications/<your_product_name_here>/<your_version>
|-- custom
|   |-- config
|   `-- source
|-- data
|-- install
|-- lasr
|-- sasmisc
`-- scenario
    `-- scenario_source</pre>

      @param t__copyfiles         <type>ArrayList&lt;Map&gt;</type>: files to copy and conditions for the copy<br>
            <indent><key>src</key>- <type>String</type>: Source directory to copy from<br>
            <key>include</key>- <type>String</type>: Regex pattern of file(s) to copy<br>
            <key>dst</key>- <type>String</type>: Destination directory for files to be copied to<br>
            <key>strip</key>- <type>String</type> <kind>optional</kind>: Remove extension form file name i.e., remove .orig extension<br>
            <key>tokens</key>- <type>String</type> <kind>optional</kind>: filterset to use for token replacement<br></indent>
      @param t__loadmetadata_createfolders   -
      @param t__importpackages_createfolders   -
      @param t__loadmetadata_load   <type>String</type>: Fully qualified path to the token substituted appxml file
      @param t__sasjobs_run      -
   */
   def properties_post =
   {
      // Simple Strings
      String offeringLegalName                     = api.'scdmid.offering.legal.name'
      String legalName                             = api.'scdmid.legal.name'
      String offeringName                          = api.'scdmid.offering.short.name'
      String normalizedOfferingLegalName           = api.'scdmid.normalized.offering.name'

      // Config Paths
      String sasConfigDir                          = api.'config.lev.applications.dir'
      String configDir                             = canonical("$sasConfigDir/$normalizedOfferingLegalName")
      String configCustCfgDir                      = canonical("$configDir/custom/config")
      String configInstallDir                      = canonical("$configDir/install")
      String autoexecPath                          = canonical("$configCustCfgDir/autoexec.sas")

      // Install Paths (Dirs and Files)
      String installConfigDir                      = api.'scdmid.install.config.dir'
      String bisrvmidinstalldir                    = api.'bisrvmid.install.dir'
    
      // Database info
      String scdmiddbmshost                        = api.'iomsrv.webinfdsvrc.host'
      String scdmiddbmsport                        = api.'iomsrv.webinfdsvrc.port'
      String scdmiddbmsname                        = api.'scdsvrc.dbms.name'

      // Properties
      api.createProperty('scdmid.config.dir', configDir)
      api.createProperty('scdmid.autoexec.path', autoexecPath)

      // Other
      String classIdentifier                       = api.'scdmid.configobj.classid'
      String majorNumber                           = api.'scdmid.major.number'
      String minorNumber                           = api.'scdmid.minor.number'
      String scdmidapphost                         = api.'vfabrcersvrc.host'
      String scdmidprotocol                        = api.'vfabrcersvrc.server.protocol'
      String scdmidport                            = api."vfabrcersvrc.server.${scdmidprotocol}.port"
      String scdmidcontext                         = api.'scdmid.contextroot'
      String signerpath                            = canonical("$bisrvmidinstalldir/Config/Deployment/Tools/signer.sh")
      String scdmiddatasourcename                  = api.'scdmid.datasource.name'
      String scdmiddatasourcenamejdbc              = "jdbc/$scdmiddatasourcename"
      String vaVersionNumber                       = api.'bihpgrdc.version'

      pass.with
      {
        _serverName                         = api.serverName()
        _appName                            = "SAS$_shortNormalizedLegalName$_releaseNumber"
        _appxmlFile                         = canonical("$configDir/install/${pass.'12byte'}.appxml")
        _applicationSubFolder               = api.'scdmid.application.sub.folder'
        _softwareComponentName              = api.'scdmid.software.component.name'
        _scdmiddbmsdataconn                = "jdbc:postgresql://$scdmiddbmshost:$scdmiddbmsport/$scdmiddbmsname"
        
        api.createProperty('scdmid.configobj.name',_softwareComponentName)

        _filters =
          [
             'class.identifier'         :  classIdentifier,
             'description'              :  _softwareComponentName,
             'product.version.folder'   :  _applicationSubFolder,
             'major.number'             :  majorNumber,
             'minor.number'             :  minorNumber,
             'software.component.name'  :  _softwareComponentName,
             'legal.name'               :  legalName,
             'release.number'           :  _releaseNumber,
             'appserver.host'           :  scdmidapphost,
             'appserver.port'           :  scdmidport,
             'appserver.protocol'       :  scdmidprotocol,
             'contextroot'              :  scdmidcontext,
             'signer.path'              :  signerpath,
             'app.name'                 :  offeringName,
             'component.name'           :  _softwareComponentName,
             'datasource.name'          :  scdmiddatasourcenamejdbc,
             'va.version.number'        :  vaVersionNumber,
             'context.name'             :  scdmidcontext,
             'product.description'      :  _softwareComponentName,
             'product.name'             :  _softwareComponentName,
             'authenticator'            :  scdmiddbmshost, 
         ]

         t__setobjectproperties_init =
         [
            properties:
            [
                'scdmid.appserver.host'         :  api.'vfabrcersvrc.host',
                'scdmid.appserver.port'         :  api."vfabrcersvrc.server.${scdmidprotocol}.port",
                'scdmid.appserver.protocol'     :  api.'vfabrcersvrc.server.protocol',
                'scdmid.dbms.host'              :  api.'iomsrv.webinfdsvrc.host',
                'scdmid.dbms.port'              :  api.'iomsrv.webinfdsvrc.port',
                'scdmid.lasr.host'              :  api.'bihpgrdc.lasrserver.host',
                'scdmid.lasr.libref'            :  api.'bihpgrdc.lasrserver.library.libref',
                'scdmid.lasr.path'              :  api.'bihpgrdc.lasrserver.path',
                'scdmid.lasr.port'              :  api.'bihpgrdc.lasrserver.port',
                'scdmid.webinfdjdbc.install.dir':  api.'webinfdjdbc.install.dir',
                'scdmid.webinfdsvrc.userid'     :  api.'iomsrv.webinfdsvrc.userid',
            ]
         ]

         t__setobjectproperties =
         [
           properties:
            [
            'scdmid.configobj.classid',
            'scdmid.configobj.version',
            'scdmid.release.number',
            'scdmid.offering.legal.name',
            'scdmid.software.component.name',
            'scdmid.appserver.host',
            'scdmid.appserver.port',
            'scdmid.appserver.protocol',
            'scdmid.dbms.host',
            'scdmid.dbms.port',
            'scdmid.lasr.host',
            'scdmid.lasr.libref',
            'scdmid.lasr.path',
            'scdmid.lasr.port',
            'scdmid.webinfdjdbc.install.dir',
            'scdmid.webinfdsvrc.userid',
            ]
         ]
                  
         t__copyfiles =
         [
            [  src:        "$installConfigDir/Deployment/Metadata",
               include:    ~/scdmid\.appxml\.orig/,
               dst:        configInstallDir,
               strip:      '.orig',
               tokens:     _filters
            ],
            [  src:        "$installConfigDir/Deployment/Metadata",
               include:   ~/.*\.locprop/,
               exclude:  ~/.*\.orig/,
               dst:         configInstallDir,
            ],
         ]

         t__loadmetadata_load = _appxmlFile

         t__setmetadataperms =
         [
            perm  : 'G',
            group : 'SASUSERS',
            set:
            [
               [
                  parentfolder: offeringLegalName,
                  folders: _softwareComponentName,
                  permNames:
                  [
                     'ReadMetadata','WriteMetadata','WriteMemberMetadata'
                  ]
               ],
            ]
         ]
         
         if (_migratedFromVersion == '6.2')
         {
            t__verifycontextroot =
            [
               fqid              :  api.'scdmid.dependson.webappsrv.fqid',
               contextroot       :  api.'scdmid.contextroot',
            ]
         }   
      }
   }

   def properties_post_datasource =
   {
      pass.with {
        t__datasource_create =
         [
            adminUser:        api.'scdmid.dbms.userid',
            adminPW:          api.decodeSASstring(api.'scdmid.dbms.passwd'),
            connectionURL:    _scdmiddbmsdataconn,
            driver:           api.'scdsvrc.dbms.data.connection.driver_class',
            datasourceName:   api.'scdsvrc.dbms.name',
            jndiName:         api.'scdmid.jndi.name',
            servername:       _serverName,
            validationQuery:  api.'scdmid.test.connection.sql',
         ]
      }
   }   

   // ===================================================== TARGET: configure

   /**
      Target: configure

      @see com.sas.gate.api.template.ConfigureTemplate
   */
   def configure()
   {
    //api.stop()

      ConfigureTemplate t = new ConfigureTemplate(this)

      t['migration > init']                     .add(configure__migration_init)
      t['properties > init']                    .add(properties_init)
      t['properties > post']                    .add(properties_post)
      t['properties > post']                    .add(properties_post_datasource)
      t['properties > post']                    .add(configure__setobjectproperties_init).when('t__migrate')
      t['loadmetadata > migration > init']      .add(configure__loadmetadata_migration_init).when('t__migrate')
      t['docparts > init']                      .add(configure__docparts_init)
      t['docparts > migration']                 .add(configure__docparts_migration).when('t__migrate')

      t.execute()

     //api.stop()
   }

   def configure__migration_init =
   {
      pass.with
      {
         t__migrate = getProperty('scdmid.migration.is_enabled', '0') { toBoolean(it) }
         
         if (t__migrate)
         {
            _migratedFromVersion  = api.'scdmid.migrated.version'
            
            t__searchandcall =
            [
               fqids:
               [
                  [
                     actions: [ __template.t__deletemetadata ],
                     type:    'Tree',
                     search:  "@Name = 'Visual Scenario Designer 6.2'"
                  ]
                ]
             ]       
         }
      }
   }
   
   def configure__loadmetadata_migration_init =
   {
      pass.with {
         if (_migratedFromVersion == '6.2')
         {
           t__loadmetadata_migration_createfolders =
           [
               _applicationSubFolder
           ]
         }
           t__loadmetadata_migration_update =
           [
               file:             _appxmlFile,
               folder:           _applicationSubFolder,
               applicationname:  _softwareComponentName
           ]
       }
   }

/**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>
   */
  def configure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
         actions:
            [
               [replace:[
                  token:'instructions.scdmid.head',
                  value:api.'instructions_scdmid_head.txt']],
               [replace:[
                  token:'instructions.scdmid.details.table.head',
                  value:api.'instructions_scdmid_details_table_head.txt']],
               [prepend:[
                  token:'instructions.scdmid.details.table.item',
                  value:api.'instructions_scdmid_details_doc.txt']],
               [replace:[
                  token:'instructions.scdmid.details.table.tail',
                  value:api.'instructions_scdmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   def configure__docparts_migration =
   {
      pass.with {
         t__docparts.actions +=
         [replace:[
            token:'instructions.scdmid.migration.report',
            value: api.'scdmid_migrateconfig_report.txt']
         ]
      }
   }

// ===================================================== TARGET: buildWebapps

     /**
        Target: buildWebapps

        @see com.sas.gate.api.template.BuildWebappsTemplate
     */


   def buildWebapps()
   {
    //api.stop()

      BuildWebappsTemplate t = new BuildWebappsTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)
      t['stage > init']             .add(stage_init)
      t['docparts > init']          .add(buildWebapps__docparts_init)

      t.execute()

    //api.stop()
   }

  def stage_init =
   {
      String configurableDir                    = canonical("$pass._scdmidinstalldir/Configurable")
      String picklistsDir                       = canonical("$pass._scdmidinstalldir/Picklists")
      String picklistFile                       = canonical("$picklistsDir/wars/$pass._rootAppName/picklist")
      String WebInfDir                          = canonical("$configurableDir/wars/$pass._rootAppName/WEB-INF")
      String classesDir                          = canonical("$configurableDir/wars/$pass._rootAppName/WEB-INF/classes")
      String MetaInfDir                         = canonical("$configurableDir/wars/$pass._rootAppName/META-INF")
      String tempDir                            = api.'config.lev.web.temp.dir'
      String tempWarDir                         = canonical("$tempDir/$pass._rootAppName")
      String description                        = api.'scdmid.description'
      String displayname                        = 'Visual Scenario Designer'
      String vsdinstalldir                      = api.'sasinstall.install.dir'

      pass.with
      {
         _filters +=
         [
              'webapp.vsd.description'        : description,
              'webapp.vsd.display.name'       : displayname,
              'webapp.vsd.installdir.name'    : vsdinstalldir,
         ]

         t__stage_extract =
         [
            retrieveFromVJR:
            [
               [
                  packageName: _rootAppName,
                  picklist:    picklistFile,
                  dst:         tempWarDir,
               ],
            ]
         ]

         t__createdirectories =
         [
            prefix: tempDir,
            entries:
            [
               "$_rootAppName/META-INF" as String,
               "$_rootAppName/$_rootAppName/WEB-INF" as String,
               "$_rootAppName/$_rootAppName/WEB-INF/classes" as String,
            ]
         ]

         t__copyfiles =
         [
            [
               src:        WebInfDir,
               include:    ~/.*\.orig/,
               dst:        "$tempWarDir/WEB-INF",
               strip:      '.orig',
               tokens:     _filters
            ],
            [
               src:        classesDir,
               include:    ~/.*\.orig/,
               dst:        "$tempWarDir/WEB-INF/classes",
               strip:      '.orig',
               tokens:     _filters
            ],
            [
               src:        MetaInfDir,
               recurse:    true,
               include:    ~/.*\.orig/,
               dst:        "$tempWarDir/META-INF",
               strip:      '.orig',
               tokens:     _filters
            ],
         ]

         t__buildarchive =
         [
            buildArchive:
            [
               [
                  appNames  : _rootAppName,
                  type      : 'war'
               ]
            ]
         ]
      }
   }

 def buildWebapps__docparts_init =
        {
           pass.with
           {
              t__docparts =
              [
                 actions:
                    [
                       [prepend:[
                       token:'instructions.scdmid.details.table.item',
                       value:api.'instructions_scdmid_details_URL.txt']],
                       [prepend:[
                       token:'instructions.scdmid.details.table.item',
                       value:api.'instructions_scdmid_details_logging.txt']],
                       [prepend:[
                       token:'instructions.scdmid.details.table.item',
                       value:api.'instructions_scdmid_details_validate.txt']],
                    ]
               ]
             }
         }
       
  // ===================================================== TARGET: deployWebapps

   /**
      Target: deployWebapps

      @see com.sas.gate.api.template.DeployWebappsTemplate
   */
  def deployWebapps()
   {
    //api.stop()

      DeployWebappsTemplate t = new DeployWebappsTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)
      t['deploywebapps > init']     .add(deploywebapps_init)

      t.execute()

     //api.stop()
   }

  def deploywebapps_init =
   {
      String stagingDir = canonical("${api.'config.lev.web.staging.dir'}/${pass._rootAppName}.war")

      pass.with {
         t__deploywebapps =
         [
            deploy:
            [
               [
                  pathtoear:  stagingDir,
                  appname:    _appName,
               ],
            ]
         ]
      }
   }
    
// ===================================================== TARGET: updateConfigure

   /**
      Target: updateConfigure

      @see com.sas.gate.api.template.ConfigureTemplate
   */
   def updateConfigure()
   {
     //api.stop()
       
       pass.with {
      _maintenanceFrom = getProperty("${pass.'12byte'}.maintenance.from.version", '')
      _fromVersion      = api.baseVersion(_maintenanceFrom)
      _maintenanceTo    = getProperty("${pass.'12byte'}.maintenance.to.version", '')
      _toVersion        = api.baseVersion(_maintenanceTo)

      if (_maintenanceFrom)
       {
         if (_maintenanceFrom != _maintenanceTo)
         {
            if (_fromVersion != _toVersion)
            {
               UpdateConfigureTemplate t = new UpdateConfigureTemplate(this)

               t['properties > pre']               .add(updateconfigure__pre_init)
               t['properties > init']              .add(properties_init)
               t['properties > post']              .add(properties_post)
               t['properties > post']              .add(updateconfigure__properties_post)
               t['loadmetadata > init']            .add(updateconfigure__loadmetadata_init)
               t['setobjectproperties > init']     .add(configure__setobjectproperties_init)
               t['docparts > init']                .add(updateconfigure__docparts_init)

               t.execute()
            }
            else
            {
               UpdateConfigureTemplate t = new UpdateConfigureTemplate(this)

               t['properties > pre']               .add(updateconfigure__pre_init)
               t['properties > init']              .add(properties_init)
               t['properties > post']              .add(properties_post)
               t['properties > post']              .add(updateconfigure__properties_post)
               t['loadmetadata > init']            .add(updateconfigure__loadmetadata_init)
               t['setobjectproperties > init']     .add(configure__setobjectproperties_init)
               t['docparts > init']                .add(updateconfigure__docparts_init)
               //We might need to add i10 doc-parts here

               t.execute()
            }
         }
      }
      else
          configure()
     } 
    //api.stop()
   }

 def updateconfigure__pre_init =
   {
       String legalName = api.createProperty('scdmid.short.legal.name', 'Visual Scenario Designer')
       String version = api.baseVersion(api.'scdmid.maintenance.to.version', api.'scdmid.release.number')
       api.createProperty('scdmid.release.number', version)
       api.createProperty('scdmid.configobj.version', version)

       String softwareComponent = api.createProperty('scdmid.software.component.name', "$legalName $version")
       String applicationFolder = api.'scdmid.application.folder'
       String oldFolderName = "$applicationFolder/$legalName 6.2"
       api.createProperty('scdmid.release.number', version)
     
       String normalizedOfferingLegalName       = api.'scdmid.normalized.offering.name'
       String sasConfigDir                      = api.'config.lev.applications.dir'
       String configDir                         = canonical("$sasConfigDir/$normalizedOfferingLegalName")
       api.createProperty('scdmid.config.dir', configDir)
     
       String subFolder = api.createProperty('scdmid.application.sub.folder', "$applicationFolder/$softwareComponent")
        
       pass.with
        {
         if ('$_fromVersion == 6.2')
          {
             t__searchandcall =
             [
               fqids:
               [
                  [
                    actions: [ __template.t__deletemetadata ],
                    type:    'Tree',
                    search:  "@Name = 'Visual Scenario Designer 6.2'"
                  ]
               ] 
             ]
          }
      }
   }   
  
  def updateconfigure__loadmetadata_init =
   {
    pass.with {
        t__loadmetadata_update =
        [
             file:             _appxmlFile,
             folder:           _applicationSubFolder,
             applicationname:  _softwareComponentName
        ]
      }
    }

  def configure__setobjectproperties_init =
    {
      pass.with {
        t__setobjectproperties +=
        [
           existingproperties: 'replace'
        ]
      }
    }

  def updateconfigure__properties_post =
   {
    pass.with {
       t__setconfigureversion =
       [
           fqid:     api.'scdmid.configobj.fqid',
           version:  _maintenanceTo
       ]
     }
   }   
   
   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def updateconfigure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
              [prepend:[
                  token:'instructions.scdmid.head',
                  value:api.'instructions_scdmid_details_table_head.txt']],
               [prepend:[
                  token:'instructions.scdmid.head',
                  value:api.'instructions_scdmid_head.txt']],
               [prepend:[
                  token:'instructions.scdmid.details.table.item',
                  value:api.'updateconfig_scdmid_details_table_item_header1.txt']],
               [prepend:[
                  token:'instructions.scdmid.details.table.item',
                  value:api.'updateconfig_scdmid_details_table_item_header2.txt']],
               [prepend:[
                  token:'instructions.scdmid.details.table.item',
                  value:api.'updateconfig_scdmid_details_table_item_header3.txt']],
               [replace:[
                  token:'instructions.scdmid.details.table.tail',
                  value:api.'instructions_scdmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   // ===================================================== TARGET: checkForErrors

   /**
      Target: checkForErrors

      @see com.sas.gate.api.template.CheckForErrorsTemplate
   */
   def checkForErrors()
   {
      CheckForErrorsTemplate t = new CheckForErrorsTemplate(this)
      t.execute()
   }

   // ===================================================== TARGET: reportFailures

   /**
      Target: reportFailures

      @see com.sas.gate.api.template.ReportFailuresTemplate
   */
   def reportFailures()
   {
      ReportFailuresTemplate t = new ReportFailuresTemplate(this)
      t.execute()
   }

   // ===================================================== TARGET: unconfigure

   /**
      Target: unconfigure

      @see com.sas.gate.api.template.UnconfigureTemplate
   */
   def unconfigure()
   {
      //api.stop()

      UnconfigureTemplate t = new UnconfigureTemplate(this)

      t['properties > init']  .add(properties_init)
      t['properties > post']  .add(properties_post)
      t['init']               .add(unconfigure__init)
      t['docparts > init']    .add(unconfigure__docparts_init)

      t.execute()

      //api.stop()
   }

   def unconfigure__init =
   {
     String stagedEar           = canonical("${api.'config.lev.web.staging.dir'}/${pass._rootAppName}.war")

      pass.with
      {
         t__removemetadatabypath =
         [
            paths:
            [
               [
                  objectPath     : _applicationSubFolder,
                  deleteContents : true
               ],
            ]
         ]

         t__removedatasource =
         [
            datasourceName:   api.'scdsvrc.dbms.name',
            servername:       _serverName
         ]

         t__undeployear =
         [
            undeploy:
            [
               [
                  earname:    _rootAppName,
                  appname:    _appName,
               ],
            ]
         ]

         t__delete_files =
         [
            paths:
            [
               stagedEar
            ]
         ]
      }
   }
   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in ConfigurationRemoval.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>file</key>- <type>String</type>: File to replace tokens in.<br>
            <key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def unconfigure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
               [replace:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.scdmid.head',
                  value:   api.'unconfiguration_summary_scdmid_head.txt']],
               [prepend:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.scdmid.item',
                  value:   api.'unconfiguration_summary_scdmid_item.txt']],
               [replace:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.scdmid.tail',
                  value:   api.'unconfiguration_summary_scdmid_tail.txt']],
            ]
         ]
      }
   }


// ===================================================== TARGET: changeHost 
   def changeHost()
   {
     //api.stop(properties:true)
     //api.runtarget(target:'changeHost.init')
      ChangeHostTemplate t = new ChangeHostTemplate(this)

      t['properties > init']  .add(properties_init)
      t['properties > post']  .add(properties_post)
      t['changehost > init']  .add(changehost__init)
      t['docparts > init']    .add(changehost__docparts_init)

      t.execute()
     //api.stop(properties:true)
   }
   
  def changehost__init =
    {
      Properties props = new Properties()
      File propsFile = new File(api.'configuration.properties.file')
      props.load(propsFile.newDataInputStream())
      api.getObjectProperties(prefix:'_!orig.',fqid:props.'scdmid.configobj.fqid')
   
      pass.with
      {
         t__changehost =
         [
            exact: ['scdmid.appserver.host':false,'scdmid.dbms.host':false,'scdmid.lasr.host':false],
          //substring: ['scdmid.dbms.connection.url'],
         ]
      }
    }

   /**
      Callback for hook: <hook>docparts &gt; init</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def changehost__docparts_init =
   {
      pass.with
      {
         if (t__changehost.docparts)
         {
            t__docparts =
            [
               file:    api.'config.lev.changehost.file',
               actions: []
            ]

            if (t__changehost.docparts.substring)
            {
               def (oldProperties, updatedProperties) = t__changehost.docparts.substring
               oldProperties.each { k, v ->

                  def value = new PipeCode([api.'changeHost_metadata_object_changed.txt'])
                     .tokenstringsub(tokens:
                     [
                        '_!PROPERTY_NAME':   k,
                        '_!OLD_VALUE':       v,
                        '_!NEW_VALUE':       updatedProperties."$k",
                     ], prefix:'${', suffix:'}').output[0]

                  t__docparts.actions += [prepend:[token: 'changeHost.configobj.item', value:value]]
               }
            }
         }
      }
   }      
   // ===================================================== TARGET: updatePasswords

   def updatePasswords()
   {
      //api.stop(properties:true)
      UpdatePasswordsTemplate t = new UpdatePasswordsTemplate(this)

      t['properties > init']              .add(properties_init)
      t['properties > post']              .add(properties_post)
      t['updatepasswords > init']         .add(updatepasswords__init)
      t['docparts > init']                .add(updatepasswords__docparts_init)

      t.execute()
     //api.stop(properties:true)
   }

   def updatepasswords__init =
   {
        
      pass.with {
         t__updatepasswords =
         [
            // Corresponds to the "array" defined by <12byte>.credentials.persisted.list
            // and that for scdmid is defined to be: scdmid.dbms
            // with the implied suffixes: .userid and .passwd
            alterwebinfdsvruser: true,
            // authdomains:         [ api.'scdsvrc.dbms.auth' ],
            createdatasource:    t__datasource_create
         ]
      }
}

   def updatepasswords__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            file:    api.'config.lev.updatepasswords.file',
            actions:
            [
               [replace:[
                  token:'updatePasswords.summary.scdmid.head',
                  value:api.'updatePasswords_scdmid_summary_head.txt']],
               [prepend:[
                  token:'updatePasswords.summary.scdmid.item',
                  value:api.'updatePasswords_scdmid_summary_item.txt']],
               [replace:[
                  token:'updatePasswords.summary.scdmid.tail',
                  value:api.'updatePasswords_scdmid_summary_tail.txt']],
            ]
         ]
      }
   }
} 
